@@ -1,4 +1,3 @@
-
 program main
     use iso_c_binding
     use spfft
@@ -15,7 +14,8 @@ program main
     integer :: errorCode = 0
     integer, dimension(dimX * dimY * dimZ * 3):: indices = 0
     complex(C_DOUBLE_COMPLEX), dimension(dimX * dimY * dimZ):: frequencyElements
-    complex(C_DOUBLE_COMPLEX), pointer :: spaceDomain(:,:,:)
+    real(C_DOUBLE), dimension(2*dimX * dimY * dimZ):: spaceDomain
+    complex(C_DOUBLE_COMPLEX), pointer :: spaceDomainPtr(:,:,:)
     type(c_ptr) :: realValuesPtr
 
 
@@ -39,9 +39,12 @@ program main
     end do
 
 
-    ! create grid and transform
+    ! create grid
     errorCode = spfft_grid_create(grid, dimX, dimY, dimZ, maxNumLocalZColumns, processingUnit, maxNumThreads);
     if (errorCode /= SPFFT_SUCCESS) error stop
+
+    ! create transform
+    ! Note: A transform handle can be created without a grid if no resource sharing is desired.
     errorCode = spfft_transform_create(transform, grid, processingUnit, 0, dimX, dimY, dimZ, dimZ,&
         size(frequencyElements), SPFFT_INDEX_TRIPLETS, indices)
     if (errorCode /= SPFFT_SUCCESS) error stop
@@ -50,6 +53,11 @@ program main
     errorCode = spfft_grid_destroy(grid)
     if (errorCode /= SPFFT_SUCCESS) error stop
 
+
+    ! *************************************************
+    ! Option A: Reuse internal buffer for space domain
+    ! *************************************************
+
     ! set space domain array to use memory allocted by the library
     errorCode = spfft_transform_get_space_domain(transform, processingUnit, realValuesPtr)
     if (errorCode /= SPFFT_SUCCESS) error stop
@@ -59,24 +67,33 @@ program main
     if (errorCode /= SPFFT_SUCCESS) error stop
 
 
-    call c_f_pointer(realValuesPtr, spaceDomain, [dimX,dimY,dimZ])
+    call c_f_pointer(realValuesPtr, spaceDomainPtr, [dimX,dimY,dimZ])
 
     print *, ""
     print *, "After backward transform:"
-    do k = 1, size(spaceDomain, 3)
-        do j = 1, size(spaceDomain, 2)
-            do i = 1, size(spaceDomain, 1)
-             print *, spaceDomain(i, j, k)
+    do k = 1, size(spaceDomainPtr, 3)
+        do j = 1, size(spaceDomainPtr, 2)
+            do i = 1, size(spaceDomainPtr, 1)
+             print *, spaceDomainPtr(i, j, k)
             end do
         end do
     end do
 
-    ! transform forward (will invalidate space domain data)
-    errorCode = spfft_transform_forward(transform, processingUnit, frequencyElements, 0)
+
+    ! **********************************************
+    ! Option B: Use external buffer for space domain
+    ! **********************************************
+
+    ! transform backward
+    errorCode = spfft_transform_backward_ptr(transform, frequencyElements, spaceDomain)
+    if (errorCode /= SPFFT_SUCCESS) error stop
+
+    ! transform forward
+    errorCode = spfft_transform_forward_ptr(transform, spaceDomain, frequencyElements, SPFFT_NO_SCALING)
     if (errorCode /= SPFFT_SUCCESS) error stop
 
     print *, ""
-    print *, "After forward transform (without scaling):"
+    print *, "After forward transform (without normalization):"
     do i = 1, size(frequencyElements)
              print *, frequencyElements(i)
     end do
